############################################
# Purpose:
# This script takes a vault and creates a namelist_precisions for that vault.
#
# Inputs:
#
#    -v, --vault : Path to the vault.
#    -o, --output-namelist : name of output namelist to specify the precision of each variable.
#
############################################
from AutoRPE.UtilsRPE.SourceManager import load_vault
from os.path import isfile


def get_command_line_arguments():
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument("-v", "--vault", default=None,
                        help="Path to the vault.")
    parser.add_argument("-o", "--output-namelist", default="namelist_precisions",
                        help="Filename for the output namelist.")

    args = parser.parse_args()

    if args.vault is None:
        parser.error("Argument -v/--vault required!")

    if not isfile(args.vault):
        parser.error("Argument -v/--vault %s is not a file." % args.vault)

    return args

def set_precision_to(value, _vault):
    """
    Function to loop over all variables where we can change the precision to set the precision value.
    """
    for variable in _vault.variables:
        if variable.mutable:
            variable.precision = value


if __name__ == "__main__":

    args = get_command_line_arguments()
    vault_path = args.vault
    output_filename = args.output_namelist

    vault = load_vault(vault_path)

    if not vault:
        print("Not able to load %s !\nUnable to create namelist!" % vault_path)
        exit(1)

    # Change the precision of all the vault variables to 52 ( the number of significant bits of a double precision real)
    set_precision_to(52, vault)

    # Generate the namelist
    vault.generate_namelist_precisions(filename=output_filename, exclude_double_precision=False)
