############################################
# Purpose:
# This script takes a vault and creates a namelist_precisions for that vault.
############################################
import AutoRPE.UtilsRPE.SourceManager as SourceManager
import AutoRPE.UtilsRPE.Inserter as Inserter
from AutoRPE.UtilsRPE.Error import PathType

from os import mkdir
from os.path import isdir
from shutil import rmtree


def get_command_line_arguments():
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument("-i", "--input-sources", required=True, type=PathType(exists=True, type='dir'),
                        help="Path to a folder that contains the preprocessed cleaned code source")
    parser.add_argument("-o", "--output-sources", default="SourcesWithRPE",
                        help="Path to the folder that will contain the AutoRPE implementation ")
    parser.add_argument("--output-vault", default="vault.pkl",
                        help="Filename of the output database")

    args = parser.parse_args()

    return args


if __name__ == "__main__":

    args = get_command_line_arguments()

    path_to_input_sources = args.input_sources
    path_to_output_sources = args.output_sources
    output_vault = args.output_vault

    # Paths to source folders
    path_to_preprocessed_sources = "./tmp_processedsources/"

    if isdir(path_to_preprocessed_sources):
        rmtree(path_to_preprocessed_sources)
    mkdir(path_to_preprocessed_sources)

    if isdir(path_to_output_sources):
        rmtree(path_to_output_sources)
    mkdir(path_to_output_sources)

    print("Starting the implementation of the reduced precision emulator in:\n   %s\n\n" % path_to_input_sources)

    # List of files that should not be processed, as red from "list_of_files_to_keep_unmodified.txt"
    preprocess_blacklist = SourceManager.load_list_of_files_to_keep_unmodified()

    # Format text + replace real declarations with type(rpe_var)
    SourceManager.replace_real_with_RPE_declaration(path_to_input_sources,
                                                    path_to_preprocessed_sources,
                                                    preprocess_blacklist)

    # Obtain source information
    vault = SourceManager.parse_sources(path_to_preprocessed_sources, save_vault=True)

    # We load the code information into the vault
    try:
        vault
    except NameError:
        vault = SourceManager.load_vault()

    # Process the sources to fix call to functions, read/write, ...
    SourceManager.postprocess_sources(path_to_preprocessed_sources, path_to_output_sources, vault=vault)

    # Insert precision specification
    Inserter.add_sbits_to_rpe_variables(path_to_output_sources, vault=vault)

    # Assign ids to variables that still don't have any (can't have it's precision changed online)
    # and set them as mutable=False
    vault.assign_ids_to_immutable_variables()

    # Add the read_precisions module into a file (should be accessible for all the other modules)
    file_to_add_read_precisions_module = None
    Inserter.insert_read_precisions_module_to_file(path_to_output_sources, file_to_add_read_precisions_module,
                                                   vault=vault)

    # Deleting temporary folders
    if isdir(path_to_preprocessed_sources):
        rmtree(path_to_preprocessed_sources)
